@@ -15,7 +15,7 @@
  */
 
 use std::collections::HashMap;
-use std::sync::{Arc, Mutex};
+use std::sync::{Arc, Mutex, RwLock};
 
 use hex_literal::hex;
 use once_cell::sync::Lazy;
@@ -23,13 +23,13 @@ use rmp_serde::{from_read, to_vec};
 use secrecy::ExposeSecret;
 use uuid::Uuid;
 
-use crate::repo::id_table::IdTable;
 use crate::store::{DataStore, OpenStore};
 
 use super::chunking::Chunking;
 use super::compression::Compression;
 use super::config::RepoConfig;
 use super::encryption::{Encryption, EncryptionKey, KeySalt, ResourceLimit};
+use super::id_table::IdTable;
 use super::lock::LockTable;
 use super::metadata::{peek_info_store, Header, RepoMetadata};
 use super::open_repo::OpenRepo;
@@ -48,10 +48,10 @@ pub const DEFAULT_INSTANCE: Uuid = Uuid::from_bytes(hex!("ea978302 bfd8 11ea b92
 ///
 /// This must be changed any time a backwards-incompatible change is made to the repository
 /// format.
-const VERSION_ID: Uuid = Uuid::from_bytes(hex!("25df72b4 8368 4409 910b fef908e1cf90"));
+const VERSION_ID: Uuid = Uuid::from_bytes(hex!("6f1c893c e6a8 11eb a198 b7fa995cc83b"));
 
 /// A table of locks on repositories.
-static REPO_LOCKS: Lazy<Mutex<LockTable>> = Lazy::new(|| Mutex::new(LockTable::new()));
+static REPO_LOCKS: Lazy<Mutex<LockTable<Uuid>>> = Lazy::new(|| Mutex::new(LockTable::new()));
 
 /// The mode to use to open a repository.
 #[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
@@ -85,10 +85,10 @@ pub enum OpenMode {
 ///
 /// let store_config = DirectoryConfig { path: "/path/to/store".into() };
 /// let mut repo: KeyRepo<String> = OpenOptions::new()
-///     .chunking(Chunking::Zpaq { bits: 18 })
+///     .chunking(Chunking::zpaq())
 ///     .compression(Compression::Lz4 { level: 1 })
 ///     .encryption(Encryption::XChaCha20Poly1305)
-///     .packing(Packing::Fixed(1024 * 16))
+///     .packing(Packing::fixed())
 ///     .password(b"password")
 ///     .mode(OpenMode::Create)
 ///     .open(&store_config)
@@ -101,10 +101,10 @@ pub enum OpenMode {
 /// use acid_store::store::DirectoryConfig;
 ///
 /// let mut repo_config = RepoConfig::default();
-/// repo_config.chunking = Chunking::Zpaq { bits: 18 };
+/// repo_config.chunking = Chunking::zpaq();
 /// repo_config.compression = Compression::Lz4 { level: 1 };
 /// repo_config.encryption = Encryption::XChaCha20Poly1305;
-/// repo_config.packing = Packing::Fixed(1024 * 16);
+/// repo_config.packing = Packing::fixed();
 ///
 /// let store_config = DirectoryConfig { path: "/path/to/store".into() };
 /// let mut repo: KeyRepo<String> = OpenOptions::new()
@@ -120,6 +120,7 @@ pub enum OpenMode {
 /// [`open`]: crate::repo::OpenOptions::open
 /// [`OpenStore`]: crate::store::OpenStore
 /// [`OpenRepo`]: crate::repo::OpenRepo
+#[derive(Debug)]
 pub struct OpenOptions {
     config: RepoConfig,
     mode: OpenMode,
@@ -127,6 +128,12 @@ pub struct OpenOptions {
     instance: Uuid,
 }
 
+impl Default for OpenOptions {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl OpenOptions {
     /// Create a new `OpenOptions`.
     pub fn new() -> Self {
@@ -256,7 +263,7 @@ impl OpenOptions {
         // Read the repository version to see if this is a compatible repository.
         let serialized_version = store
             .read_block(VERSION_BLOCK_ID)
-            .map_err(|error| crate::Error::Store(error))?
+            .map_err(crate::Error::Store)?
             .ok_or(crate::Error::NotFound)?;
         let version =
             Uuid::from_slice(serialized_version.as_slice()).map_err(|_| crate::Error::Corrupt)?;
@@ -268,7 +275,7 @@ impl OpenOptions {
         // acquiring the lock.
         let serialized_metadata = store
             .read_block(METADATA_BLOCK_ID)
-            .map_err(|error| crate::Error::Store(error))?
+            .map_err(crate::Error::Store)?
             .ok_or(crate::Error::Corrupt)?;
         let metadata: RepoMetadata =
             from_read(serialized_metadata.as_slice()).map_err(|_| crate::Error::Corrupt)?;
@@ -306,7 +313,7 @@ impl OpenOptions {
         // Read, decrypt, decompress, and deserialize the repository header.
         let encrypted_header = store
             .read_block(metadata.header_id)
-            .map_err(|error| crate::Error::Store(error))?
+            .map_err(crate::Error::Store)?
             .ok_or(crate::Error::Corrupt)?;
         let compressed_header = metadata
             .config
@@ -327,14 +334,15 @@ impl OpenOptions {
             handle_table,
         } = header;
 
-        let state = RepoState {
+        let state = Arc::new(RwLock::new(RepoState {
             store: Mutex::new(Box::new(store)),
             metadata,
             chunks,
             packs,
+            transactions: LockTable::new(),
             master_key,
             lock,
-        };
+        }));
 
         let repo: KeyRepo<R::Key> = KeyRepo {
             state,
@@ -345,7 +353,7 @@ impl OpenOptions {
             transaction_id: Arc::new(Uuid::new_v4()),
         };
 
-        repo.set_instance(self.instance)
+        repo.change_instance(self.instance)
     }
 
     /// Create a new repository, failing if one already exists.
@@ -370,7 +378,7 @@ impl OpenOptions {
         // Check if the repository already exists.
         if store
             .read_block(VERSION_BLOCK_ID)
-            .map_err(|error| crate::Error::Store(error))?
+            .map_err(crate::Error::Store)?
             .is_some()
         {
             return Err(crate::Error::AlreadyExists);
@@ -423,7 +431,7 @@ impl OpenOptions {
         let header_id = Uuid::new_v4();
         store
             .write_block(header_id, &encrypted_header)
-            .map_err(|error| crate::Error::Store(error))?;
+            .map_err(crate::Error::Store)?;
 
         // Create the repository metadata with the header block references.
         let metadata = RepoMetadata {
@@ -438,13 +446,13 @@ impl OpenOptions {
         let serialized_metadata = to_vec(&metadata).expect("Could not serialize metadata.");
         store
             .write_block(METADATA_BLOCK_ID, &serialized_metadata)
-            .map_err(|error| crate::Error::Store(error))?;
+            .map_err(crate::Error::Store)?;
 
         // Write the repository version. We do this last because this signifies that the repository
         // is done being created.
         store
             .write_block(VERSION_BLOCK_ID, VERSION_ID.as_bytes())
-            .map_err(|error| crate::Error::Store(error))?;
+            .map_err(crate::Error::Store)?;
 
         let Header {
             chunks,
@@ -453,14 +461,15 @@ impl OpenOptions {
             handle_table,
         } = header;
 
-        let state = RepoState {
+        let state = Arc::new(RwLock::new(RepoState {
             store: Mutex::new(Box::new(store)),
             metadata,
             chunks,
             packs,
+            transactions: LockTable::new(),
             master_key,
             lock,
-        };
+        }));
 
         let repo: KeyRepo<R::Key> = KeyRepo {
             state,
@@ -471,7 +480,7 @@ impl OpenOptions {
             transaction_id: Arc::new(Uuid::new_v4()),
         };
 
-        repo.set_instance(self.instance)
+        repo.change_instance(self.instance)
     }
 
     /// Open or create the repository.
@@ -507,7 +516,7 @@ impl OpenOptions {
             OpenMode::Create => {
                 if store
                     .read_block(VERSION_BLOCK_ID)
-                    .map_err(|error| crate::Error::Store(error))?
+                    .map_err(crate::Error::Store)?
                     .is_some()
                 {
                     self.open_repo(store)
