@@ -20,21 +20,21 @@ use std::collections::HashMap;
 use std::fs::{create_dir, File};
 use std::io::{Read, Write};
 
+#[cfg(all(target_os = "linux", feature = "file-metadata"))]
+use exacl::{AclEntry, AclEntryKind, AclOption, Flag, Perm};
 use maplit::hashmap;
-#[cfg(all(linux, feature = "file-metadata"))]
-use posix_acl::{PosixACL, Qualifier as PosixQualifier};
 use relative_path::RelativePathBuf;
 use tempfile::tempdir;
 
 use acid_store::repo::file::{Entry, FileRepo, NoMetadata, NoSpecialType};
-use acid_store::repo::{OpenMode, OpenOptions, SwitchInstance, DEFAULT_INSTANCE};
+use acid_store::repo::{Commit, OpenMode, OpenOptions, SwitchInstance, DEFAULT_INSTANCE};
 use acid_store::store::MemoryConfig;
 use acid_store::uuid::Uuid;
 use common::{assert_contains_all, random_buffer};
 #[cfg(all(unix, feature = "file-metadata"))]
 use {
     acid_store::repo::file::{
-        AccessQualifier, CommonMetadata, FileType, UnixMetadata, UnixSpecialType,
+        AccessMode, AccessQualifier, Acl, CommonMetadata, EntryType, UnixMetadata, UnixSpecialType,
     },
     nix::sys::stat::{Mode, SFlag},
     nix::unistd::mkfifo,
@@ -66,9 +66,9 @@ fn switching_instance_does_not_roll_back() -> anyhow::Result<()> {
     let mut repo = create_repo(&config)?;
 
     repo.create("file", &Entry::file())?;
-    let mut object = repo.open_mut("file")?;
+    let mut object = repo.open("file")?;
     object.write_all(random_buffer().as_slice())?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     let repo: FileRepo = repo.switch_instance(Uuid::new_v4())?;
@@ -86,9 +86,9 @@ fn switching_instance_does_not_commit() -> anyhow::Result<()> {
     let mut repo = create_repo(&config)?;
 
     repo.create("file", &Entry::file())?;
-    let mut object = repo.open_mut("file")?;
+    let mut object = repo.open("file")?;
     object.write_all(random_buffer().as_slice())?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     let repo: FileRepo = repo.switch_instance(Uuid::new_v4())?;
@@ -104,6 +104,19 @@ fn switching_instance_does_not_commit() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn empty_path_does_not_exist() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let repository = create_repo(&config)?;
+
+    assert!(!repository.exists(""));
+    assert!(!repository.is_file(""));
+    assert!(!repository.is_directory(""));
+    assert!(!repository.is_special(""));
+
+    Ok(())
+}
+
 #[test]
 fn creating_existing_file_errs() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -126,17 +139,31 @@ fn creating_file_without_parent_errs() -> anyhow::Result<()> {
 
     // Creating a directory without a parent fails.
     let result = repository.create("home/lostatc", &Entry::directory());
-    assert!(matches!(
-        result.unwrap_err(),
-        acid_store::Error::InvalidPath
-    ));
+    assert!(matches!(result.unwrap_err(), acid_store::Error::NotFound));
 
     // Creating a directory as a child of a file fails.
     repository.create("home", &Entry::file())?;
     let result = repository.create("home/lostatc", &Entry::directory());
     assert!(matches!(
         result.unwrap_err(),
-        acid_store::Error::InvalidPath
+        acid_store::Error::NotDirectory
+    ));
+
+    Ok(())
+}
+
+#[test]
+fn creating_empty_path_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.create("", &Entry::file()),
+        Err(acid_store::Error::InvalidPath)
+    ));
+    assert!(matches!(
+        repository.create_parents("", &Entry::file()),
+        Err(acid_store::Error::InvalidPath)
     ));
 
     Ok(())
@@ -188,6 +215,23 @@ fn removing_non_empty_directory_errs() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn removing_emtpy_path_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.remove(""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+    assert!(matches!(
+        repository.remove_tree(""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 fn remove_file() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -223,6 +267,19 @@ fn remove_tree_without_descendants() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn getting_empty_path_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.entry(""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 fn setting_metadata_on_nonexistent_file_errs() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -233,6 +290,19 @@ fn setting_metadata_on_nonexistent_file_errs() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn setting_metadata_on_empty_path_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.set_metadata("", None),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 #[cfg(feature = "file-metadata")]
 fn set_metadata() -> anyhow::Result<()> {
@@ -257,10 +327,10 @@ fn open_file() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
     let mut repository = create_repo(&config)?;
     repository.create("file", &Entry::file())?;
-    let mut object = repository.open_mut("file")?;
+    let mut object = repository.open("file")?;
 
     object.write_all(b"expected data")?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     let mut object = repository.open("file")?;
@@ -271,6 +341,19 @@ fn open_file() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn opening_empty_path_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.open(""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 fn copied_file_has_same_contents() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -281,9 +364,9 @@ fn copied_file_has_same_contents() -> anyhow::Result<()> {
 
     // Add a file entry and write data to it.
     repository.create("source", &Entry::file())?;
-    let mut object = repository.open_mut("source")?;
+    let mut object = repository.open("source")?;
     object.write_all(expected_data)?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     // Copy the file entry.
@@ -304,10 +387,17 @@ fn copy_file_with_invalid_destination() -> anyhow::Result<()> {
 
     assert!(matches!(
         repository.copy("source", "nonexistent/dest"),
-        Err(acid_store::Error::InvalidPath)
+        Err(acid_store::Error::NotFound)
     ));
 
-    repository.create("dest", &Entry::file())?;
+    repository.create("file", &Entry::file())?;
+
+    assert!(matches!(
+        repository.copy("source", "file/dest"),
+        Err(acid_store::Error::NotDirectory)
+    ));
+
+    repository.create("dest", &Entry::directory())?;
 
     assert!(matches!(
         repository.copy("source", "dest"),
@@ -332,6 +422,19 @@ fn copy_tree() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn copy_subdirectory_tree() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+    repository.create_parents("root/source/file1", &Entry::file())?;
+
+    repository.copy_tree("root/source", "root/dest")?;
+
+    assert!(repository.entry("root/dest/file1")?.is_file());
+
+    Ok(())
+}
+
 #[test]
 fn copy_tree_which_is_a_file() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -345,6 +448,42 @@ fn copy_tree_which_is_a_file() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn copying_with_empty_path_as_source_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.copy("", "test"),
+        Err(acid_store::Error::InvalidPath)
+    ));
+    assert!(matches!(
+        repository.copy_tree("", "test"),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
+#[test]
+fn copying_with_empty_path_as_dest_errs() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+
+    repository.create("test", &Entry::file())?;
+
+    assert!(matches!(
+        repository.copy("test", ""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+    assert!(matches!(
+        repository.copy_tree("test", ""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 fn list_children() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -386,6 +525,24 @@ fn list_children_of_a_file() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn list_children_of_empty_path() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+    repository.create("file1", &Entry::file())?;
+    repository.create("file2", &Entry::file())?;
+
+    let actual = repository.list("")?;
+    let expected = vec![
+        RelativePathBuf::from("file1"),
+        RelativePathBuf::from("file2"),
+    ];
+
+    assert_contains_all(actual, expected);
+
+    Ok(())
+}
+
 #[test]
 fn walk_descendants() -> anyhow::Result<()> {
     let config = MemoryConfig::new();
@@ -428,6 +585,23 @@ fn walk_descendants_of_a_file() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn walk_descendants_of_empty_path() -> anyhow::Result<()> {
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+    repository.create_parents("directory/file", &Entry::file())?;
+
+    let actual = repository.walk("")?;
+    let expected = vec![
+        RelativePathBuf::from("directory"),
+        RelativePathBuf::from("directory/file"),
+    ];
+
+    assert_contains_all(actual, expected);
+
+    Ok(())
+}
+
 #[test]
 fn archive_file() -> anyhow::Result<()> {
     let temp_dir = tempdir()?;
@@ -472,16 +646,16 @@ fn archive_unix_special_files() -> anyhow::Result<()> {
     let symlink_entry = repository.entry("dest/symlink")?;
     let device_entry = repository.entry("dest/device")?;
 
-    assert_eq!(fifo_entry.file_type, UnixSpecialType::NamedPipe.into());
+    assert_eq!(fifo_entry.kind, UnixSpecialType::NamedPipe.into());
     assert_eq!(
-        symlink_entry.file_type,
+        symlink_entry.kind,
         UnixSpecialType::SymbolicLink {
             target: "/dev/null".into()
         }
         .into()
     );
     assert_eq!(
-        device_entry.file_type,
+        device_entry.kind,
         UnixSpecialType::CharacterDevice { major: 1, minor: 3 }.into()
     );
     Ok(())
@@ -523,6 +697,27 @@ fn archive_tree() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn archiving_to_empty_path_errs() -> anyhow::Result<()> {
+    let temp_dir = tempdir()?;
+    let source_path = temp_dir.as_ref().join("source");
+    File::create(&source_path)?;
+
+    let config = MemoryConfig::new();
+    let mut repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.archive(&source_path, ""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+    assert!(matches!(
+        repository.archive_tree(&source_path, ""),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 fn extract_file() -> anyhow::Result<()> {
     let temp_dir = tempdir()?;
@@ -532,9 +727,9 @@ fn extract_file() -> anyhow::Result<()> {
     let mut repository = create_repo(&config)?;
 
     repository.create("source", &Entry::file())?;
-    let mut object = repository.open_mut("source")?;
+    let mut object = repository.open("source")?;
     object.write_all(b"file contents")?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
     repository.extract("source", &dest_path)?;
 
@@ -628,6 +823,26 @@ fn extract_tree() -> anyhow::Result<()> {
     Ok(())
 }
 
+#[test]
+fn extracting_from_empty_path_errs() -> anyhow::Result<()> {
+    let temp_dir = tempdir()?;
+    let dest_path = temp_dir.as_ref().join("dest");
+
+    let config = MemoryConfig::new();
+    let repository = create_repo(&config)?;
+
+    assert!(matches!(
+        repository.extract("", &dest_path),
+        Err(acid_store::Error::InvalidPath)
+    ));
+    assert!(matches!(
+        repository.extract_tree("", &dest_path),
+        Err(acid_store::Error::InvalidPath)
+    ));
+
+    Ok(())
+}
+
 #[test]
 #[cfg(all(unix, feature = "file-metadata"))]
 fn write_unix_metadata() -> anyhow::Result<()> {
@@ -642,16 +857,20 @@ fn write_unix_metadata() -> anyhow::Result<()> {
     // on tmpfs, which is most likely where the temporary directory will be created.
 
     let entry_metadata = UnixMetadata {
-        mode: 0o666,
+        mode: 0o246,
         modified: SystemTime::UNIX_EPOCH,
         accessed: SystemTime::UNIX_EPOCH,
+        changed: SystemTime::UNIX_EPOCH,
         user: 1000,
         group: 1000,
         attributes: HashMap::new(),
-        acl: hashmap! { AccessQualifier::User(1001) => 0o777 },
+        acl: Acl {
+            access: hashmap! { AccessQualifier::User(65533) => AccessMode::READ | AccessMode::WRITE | AccessMode::EXECUTE },
+            default: HashMap::new(),
+        },
     };
     let entry = Entry {
-        file_type: FileType::File,
+        kind: EntryType::File,
         metadata: Some(entry_metadata.clone()),
     };
 
@@ -666,10 +885,51 @@ fn write_unix_metadata() -> anyhow::Result<()> {
     assert_eq!(dest_metadata.modified()?, entry_metadata.modified);
     assert_eq!(dest_metadata.accessed()?, entry_metadata.accessed);
 
-    #[cfg(linux)]
+    #[cfg(target_os = "linux")]
     {
-        let dest_acl = PosixACL::new(dest_metadata.mode());
-        assert_eq!(dest_acl.get(PosixQualifier::User(1001)), Some(0o777));
+        let actual_entries = exacl::getfacl(dest_path, AclOption::ACCESS_ACL)?;
+        let user_entry = AclEntry {
+            kind: AclEntryKind::User,
+            name: String::new(),
+            perms: Perm::WRITE,
+            flags: Flag::empty(),
+            allow: true,
+        };
+        let group_entry = AclEntry {
+            kind: AclEntryKind::Group,
+            name: String::new(),
+            perms: Perm::READ,
+            flags: Flag::empty(),
+            allow: true,
+        };
+        let other_entry = AclEntry {
+            kind: AclEntryKind::Other,
+            name: String::new(),
+            perms: Perm::READ | Perm::WRITE,
+            flags: Flag::empty(),
+            allow: true,
+        };
+        let mask_entry = AclEntry {
+            kind: AclEntryKind::Mask,
+            name: String::new(),
+            perms: Perm::READ | Perm::WRITE | Perm::EXECUTE,
+            flags: Flag::empty(),
+            allow: true,
+        };
+        let new_entry = AclEntry {
+            kind: AclEntryKind::User,
+            name: "65533".to_string(),
+            perms: Perm::READ | Perm::WRITE | Perm::EXECUTE,
+            flags: Flag::empty(),
+            allow: true,
+        };
+
+        assert_eq!(actual_entries.len(), 5);
+        assert!(actual_entries.contains(&user_entry));
+        assert!(actual_entries.contains(&group_entry));
+        assert!(actual_entries.contains(&other_entry));
+        assert!(actual_entries.contains(&mask_entry));
+        assert!(actual_entries.contains(&new_entry));
     }
 
     Ok(())
@@ -681,12 +941,19 @@ fn read_unix_metadata() -> anyhow::Result<()> {
     let temp_dir = tempdir()?;
     let source_path = temp_dir.as_ref().join("source");
     File::create(&source_path)?;
+    let source_metadata = source_path.metadata()?;
 
-    #[cfg(linux)]
+    #[cfg(target_os = "linux")]
     {
-        let mut dest_acl = PosixACL::new(source_path.metadata()?.mode());
-        dest_acl.set(PosixQualifier::User(1001), 0o777);
-        dest_acl.write_acl(&source_path)?;
+        let mut entries = exacl::getfacl(&source_path, AclOption::ACCESS_ACL)?;
+        entries.push(AclEntry {
+            kind: AclEntryKind::User,
+            name: "65533".to_string(),
+            perms: Perm::READ | Perm::WRITE | Perm::EXECUTE,
+            flags: Flag::empty(),
+            allow: true,
+        });
+        exacl::setfacl(&[&source_path], &entries, AclOption::ACCESS_ACL)?;
     }
 
     let config = MemoryConfig::new();
@@ -696,7 +963,6 @@ fn read_unix_metadata() -> anyhow::Result<()> {
     repository.archive(&source_path, "dest")?;
     let entry = repository.entry("dest")?;
     let entry_metadata = entry.metadata.unwrap();
-    let source_metadata = source_path.metadata()?;
 
     // This does not test extended attributes because user extended attributes are not supported
     // on tmpfs, which is most likely where the temporary directory will be created.
@@ -706,11 +972,15 @@ fn read_unix_metadata() -> anyhow::Result<()> {
     assert_eq!(entry_metadata.user, source_metadata.uid());
     assert_eq!(entry_metadata.group, source_metadata.gid());
 
-    #[cfg(linux)]
+    #[cfg(target_os = "linux")]
     {
         assert_eq!(
-            entry_metadata.acl,
-            hashmap! { AccessQualifier::User(1001) => 0o777 }
+            entry_metadata
+                .acl
+                .access
+                .get(&AccessQualifier::User(65533))
+                .copied(),
+            Some(AccessMode::READ | AccessMode::WRITE | AccessMode::EXECUTE)
         );
     }
 
@@ -732,7 +1002,7 @@ fn write_common_metadata() -> anyhow::Result<()> {
         accessed: SystemTime::UNIX_EPOCH,
     };
     let entry = Entry {
-        file_type: FileType::File,
+        kind: EntryType::File,
         metadata: Some(entry_metadata.clone()),
     };
 
@@ -773,9 +1043,9 @@ fn entries_removed_on_rollback() -> anyhow::Result<()> {
     let mut repository = create_repo(&config)?;
     repository.create("file", &Entry::file())?;
 
-    let mut object = repository.open_mut("file")?;
+    let mut object = repository.open("file")?;
     object.write_all(random_buffer().as_slice())?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     repository.rollback()?;
@@ -791,9 +1061,9 @@ fn clear_instance_removes_paths() -> anyhow::Result<()> {
     let mut repo = create_repo(&config)?;
 
     repo.create("test", &Entry::file())?;
-    let mut object = repo.open_mut("test")?;
+    let mut object = repo.open("test")?;
     object.write_all(random_buffer().as_slice())?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     repo.clear_instance();
@@ -813,9 +1083,9 @@ fn rollback_after_clear_instance() -> anyhow::Result<()> {
     let mut repo = create_repo(&config)?;
 
     repo.create("test", &Entry::file())?;
-    let mut object = repo.open_mut("test")?;
+    let mut object = repo.open("test")?;
     object.write_all(random_buffer().as_slice())?;
-    object.flush()?;
+    object.commit()?;
     drop(object);
 
     repo.commit()?;
